@@ -1,29 +1,52 @@
-"""Command line tool for managing tmux workspaces and tmuxp configurations.
+"""Command line tool for managing tmuxp workspaces.
 
-tmuxp.cli
-~~~~~~~~~
+tmuxp.cli.load
+~~~~~~~~~~~~~~
 
 """
+import argparse
 import importlib
 import logging
 import os
 import pathlib
 import shutil
 import sys
-from typing import List
-
-import click
+import typing as t
 
 from libtmux.common import has_gte_version
 from libtmux.server import Server
-from tmuxp import config_reader
+from libtmux.session import Session
+from tmuxp.types import StrPath
 
-from .. import config, exc, log, util
-from ..workspacebuilder import WorkspaceBuilder
-from .utils import ConfigPath, _validate_choices, get_config_dir, tmuxp_echo
+from .. import config_reader, exc, log, util
+from ..workspace import loader
+from ..workspace.builder import WorkspaceBuilder
+from ..workspace.finders import find_workspace_file, get_workspace_dir, in_dir
+from .utils import prompt_choices, prompt_yes_no, style, tmuxp_echo
+
+if t.TYPE_CHECKING:
+    from typing_extensions import Literal, NotRequired, TypeAlias, TypedDict
+
+    CLIColorsLiteral: TypeAlias = Literal[56, 88]
+
+    class OptionOverrides(TypedDict):
+        detached: NotRequired[bool]
+        new_session_name: NotRequired[t.Optional[str]]
 
 
-def set_layout_hook(session, hook_name):
+class CLILoadNamespace(argparse.Namespace):
+    workspace_files: t.List[str]
+    socket_name: t.Optional[str]
+    socket_path: t.Optional[str]
+    tmux_config_file: t.Optional[str]
+    new_session_name: t.Optional[str]
+    answer_yes: t.Optional[bool]
+    append: t.Optional[bool]
+    colors: t.Optional["CLIColorsLiteral"]
+    log_file: t.Optional[str]
+
+
+def set_layout_hook(session: Session, hook_name: str) -> None:
     """Set layout hooks to normalize layout.
 
     References:
@@ -75,18 +98,18 @@ def set_layout_hook(session, hook_name):
     hook_cmd.append(f"selectw -t {attached_window.id}")
 
     # join the hook's commands with semicolons
-    hook_cmd = "{}".format("; ".join(hook_cmd))
+    _hook_cmd = "{}".format("; ".join(hook_cmd))
 
     # append the hook command
-    cmd.append(hook_cmd)
+    cmd.append(_hook_cmd)
 
     # create the hook
     session.cmd(*cmd)
 
 
-def load_plugins(sconf):
+def load_plugins(sconf: t.Any) -> t.List[t.Any]:
     """
-    Load and return plugins in config
+    Load and return plugins in workspace
     """
     plugins = []
     if "plugins" in sconf:
@@ -98,34 +121,34 @@ def load_plugins(sconf):
                 plugin = getattr(importlib.import_module(module_name), plugin_name)
                 plugins.append(plugin())
             except exc.TmuxpPluginException as error:
-                if not click.confirm(
+                if not prompt_yes_no(
                     "%sSkip loading %s?"
-                    % (click.style(str(error), fg="yellow"), plugin_name),
+                    % (style(str(error), fg="yellow"), plugin_name),
                     default=True,
                 ):
-                    click.echo(
-                        click.style("[Not Skipping] ", fg="yellow")
+                    tmuxp_echo(
+                        style("[Not Skipping] ", fg="yellow")
                         + "Plugin versions constraint not met. Exiting..."
                     )
                     sys.exit(1)
             except Exception as error:
-                click.echo(
-                    click.style("[Plugin Error] ", fg="red")
+                tmuxp_echo(
+                    style("[Plugin Error] ", fg="red")
                     + f"Couldn't load {plugin}\n"
-                    + click.style(f"{error}", fg="yellow")
+                    + style(f"{error}", fg="yellow")
                 )
                 sys.exit(1)
 
     return plugins
 
 
-def _reattach(builder):
+def _reattach(builder: WorkspaceBuilder):
     """
     Reattach session (depending on env being inside tmux already or not)
 
     Parameters
     ----------
-    builder: :class:`workspacebuilder.WorkspaceBuilder`
+    builder: :class:`workspace.builder.WorkspaceBuilder`
 
     Notes
     -----
@@ -148,13 +171,13 @@ def _reattach(builder):
         builder.session.attach_session()
 
 
-def _load_attached(builder, detached):
+def _load_attached(builder: WorkspaceBuilder, detached: bool) -> None:
     """
-    Load config in new session
+    Load workspace in new session
 
     Parameters
     ----------
-    builder: :class:`workspacebuilder.WorkspaceBuilder`
+    builder: :class:`workspace.builder.WorkspaceBuilder`
     detached : bool
     """
     builder.build()
@@ -181,13 +204,13 @@ def _load_attached(builder, detached):
             builder.session.attach_session()
 
 
-def _load_detached(builder):
+def _load_detached(builder: WorkspaceBuilder) -> None:
     """
-    Load config in new session but don't attach
+    Load workspace in new session but don't attach
 
     Parameters
     ----------
-    builder: :class:`workspacebuilder.WorkspaceBuilder`
+    builder: :class:`workspace.builder.WorkspaceBuilder`
     """
     builder.build()
 
@@ -198,13 +221,13 @@ def _load_detached(builder):
     print("Session created in detached state.")
 
 
-def _load_append_windows_to_current_session(builder):
+def _load_append_windows_to_current_session(builder: WorkspaceBuilder) -> None:
     """
-    Load config as new windows in current session
+    Load workspace as new windows in current session
 
     Parameters
     ----------
-    builder: :class:`workspacebuilder.WorkspaceBuilder`
+    builder: :class:`workspace.builder.WorkspaceBuilder`
     """
     current_attached_session = builder.find_current_attached_session()
     builder.build(current_attached_session, append=True)
@@ -213,13 +236,13 @@ def _load_append_windows_to_current_session(builder):
         set_layout_hook(builder.session, "client-session-changed")
 
 
-def _setup_plugins(builder):
+def _setup_plugins(builder: WorkspaceBuilder) -> Session:
     """
     Runs after before_script
 
     Parameters
     ----------
-    builder: :class:`workspacebuilder.WorkspaceBuilder`
+    builder: :class:`workspace.builder.WorkspaceBuilder`
     """
     for plugin in builder.plugins:
         plugin.before_script(builder.session)
@@ -228,23 +251,23 @@ def _setup_plugins(builder):
 
 
 def load_workspace(
-    config_file,
-    socket_name=None,
-    socket_path=None,
-    tmux_config_file=None,
-    new_session_name=None,
-    colors=None,
-    detached=False,
-    answer_yes=False,
-    append=False,
-):
+    workspace_file: StrPath,
+    socket_name: t.Optional[str] = None,
+    socket_path: None = None,
+    tmux_config_file: None = None,
+    new_session_name: t.Optional[str] = None,
+    colors: t.Optional[int] = None,
+    detached: bool = False,
+    answer_yes: bool = False,
+    append: bool = False,
+) -> t.Optional[Session]:
     """
     Load a tmux "workspace" session via tmuxp file.
 
     Parameters
     ----------
-    config_file : str
-        absolute path to config file
+    workspace_file : list of str
+        paths or session names to workspace files
     socket_name : str, optional
         ``tmux -L <socket-name>``
     socket_path: str, optional
@@ -266,33 +289,33 @@ def load_workspace(
     Notes
     -----
 
-    tmuxp will check and load a configuration file. The file will use ConfigReader
-    to load a JSON/YAML into a :py:obj:`dict`. Then :func:`config.expand` and
-    :func:`config.trickle` will be used to expand any shorthands, template
+    tmuxp will check and load a workspace file. The file will use ConfigReader
+    to load a JSON/YAML into a :py:obj:`dict`. Then :func:`loader.expand` and
+    :func:`loader.trickle` will be used to expand any shorthands, template
     variables, or file paths relative to where the config/script is executed
     from.
 
-    :func:`config.expand` accepts the directory of the config file, so the
-    user's configuration can resolve absolute paths relative to where the
-    config file is. In otherwords, if a config file at */var/moo/hi.yaml*
-    has *./* in its configs, we want to be sure any file path with *./* is
+    :func:`loader.expand` accepts the directory of the config file, so the
+    user's workspace can resolve absolute paths relative to where the
+    workspace file is. In otherwords, if a workspace file at */var/moo/hi.yaml*
+    has *./* in its workspaces, we want to be sure any file path with *./* is
     relative to */var/moo*, not the user's PWD.
 
     A :class:`libtmux.Server` object is created. No tmux server is started yet,
     just the object.
 
-    The prepared configuration and the server object is passed into an instance
-    of :class:`~tmuxp.workspacebuilder.WorkspaceBuilder`.
+    The prepared workspace and its server object is passed into an instance
+    of :class:`~tmuxp.workspace.builder.WorkspaceBuilder`.
 
     A sanity check against :meth:`libtmux.common.which` is ran. It will raise
     an exception if tmux isn't found.
 
     If a tmux session under the same name as ``session_name`` in the tmuxp
-    configuration exists, tmuxp offers to attach the session. Currently, tmuxp
+    workspace exists, tmuxp offers to attach the session. Currently, tmuxp
     does not allow appending a workspace / incremental building on top of a
     current session (pull requests are welcome!).
 
-    :meth:`~tmuxp.workspacebuilder.WorkspaceBuilder.build` will build the session in
+    :meth:`~tmuxp.workspace.builder.build` will build the session in
     the background via using tmux's detached state (``-d``).
 
     After the session (workspace) is built, unless the user decided to load
@@ -333,55 +356,59 @@ def load_workspace(
        Accessed April 8th, 2018.
     """
     # get the canonical path, eliminating any symlinks
-    if isinstance(config_file, str):
-        config_file = pathlib.Path(config_file)
+    if isinstance(workspace_file, (str, os.PathLike)):
+        workspace_file = pathlib.Path(workspace_file)
 
     tmuxp_echo(
-        click.style("[Loading] ", fg="green")
-        + click.style(str(config_file), fg="blue", bold=True)
+        style("[Loading] ", fg="green")
+        + style(str(workspace_file), fg="blue", bold=True)
     )
 
     # ConfigReader allows us to open a yaml or json file as a dict
-    raw_config = config_reader.ConfigReader._from_file(config_file)
-    # shapes configurations relative to config / profile file location
-    sconfig = config.expand(raw_config, cwd=os.path.dirname(config_file))
-    # Overwrite session name
+    raw_workspace = config_reader.ConfigReader._from_file(workspace_file) or {}
+
+    # shapes workspaces relative to config / profile file location
+    expanded_workspace = loader.expand(
+        raw_workspace, cwd=os.path.dirname(workspace_file)
+    )
+
+    # Overridden session name
     if new_session_name:
-        sconfig["session_name"] = new_session_name
-    # propagate config inheritance (e.g. session -> window, window -> pane)
-    sconfig = config.trickle(sconfig)
+        expanded_workspace["session_name"] = new_session_name
+
+    # propagate workspace inheritance (e.g. session -> window, window -> pane)
+    expanded_workspace = loader.trickle(expanded_workspace)
 
     t = Server(  # create tmux server object
         socket_name=socket_name,
         socket_path=socket_path,
-        config_file=tmux_config_file,
+        workspace_file=tmux_config_file,
         colors=colors,
     )
 
     shutil.which("tmux")  # raise exception if tmux not found
 
-    try:  # load WorkspaceBuilder object for tmuxp config / tmux server
+    try:  # load WorkspaceBuilder object for tmuxp workspace / tmux server
         builder = WorkspaceBuilder(
-            sconf=sconfig, plugins=load_plugins(sconfig), server=t
+            sconf=expanded_workspace, plugins=load_plugins(expanded_workspace), server=t
         )
-    except exc.EmptyConfigException:
-        tmuxp_echo("%s is empty or parsed no config data" % config_file, err=True)
-        return
+    except exc.EmptyWorkspaceException:
+        tmuxp_echo("%s is empty or parsed no workspace data" % workspace_file)
+        return None
 
-    session_name = sconfig["session_name"]
+    session_name = expanded_workspace["session_name"]
 
     # if the session already exists, prompt the user to attach
     if builder.session_exists(session_name) and not append:
         if not detached and (
             answer_yes
-            or click.confirm(
-                "%s is already running. Attach?"
-                % click.style(session_name, fg="green"),
+            or prompt_yes_no(
+                "%s is already running. Attach?" % style(session_name, fg="green"),
                 default=True,
             )
         ):
             _reattach(builder)
-        return
+        return None
 
     try:
         if detached:
@@ -407,7 +434,7 @@ def load_workspace(
                 "Or (a)ppend windows in the current active session?\n[y/n/a]"
             )
             options = ["y", "n", "a"]
-            choice = click.prompt(msg, value_proc=_validate_choices(options))
+            choice = prompt_choices(msg, choices=options)
 
             if choice == "y":
                 _load_attached(builder, detached)
@@ -421,12 +448,12 @@ def load_workspace(
     except exc.TmuxpException as e:
         import traceback
 
-        tmuxp_echo(traceback.format_exc(), err=True)
-        tmuxp_echo(e, err=True)
+        tmuxp_echo(traceback.format_exc())
+        tmuxp_echo(str(e))
 
-        choice = click.prompt(
+        choice = prompt_choices(
             "Error loading workspace. (k)ill, (a)ttach, (d)etach?",
-            value_proc=_validate_choices(["k", "a", "d"]),
+            choices=["k", "a", "d"],
             default="k",
         )
 
@@ -441,15 +468,15 @@ def load_workspace(
     return _setup_plugins(builder)
 
 
-def config_file_completion(ctx, params, incomplete):
-    config_dir = pathlib.Path(get_config_dir())
-    choices: List[pathlib.Path] = []
+def workspace_file_completion(ctx, params, incomplete):
+    workspace_dir = pathlib.Path(get_workspace_dir())
+    choices: t.List[pathlib.Path] = []
 
     # CWD Paths
     choices += sorted(
         pathlib.Path(os.path.relpath(p, pathlib.Path.cwd()))
         for p in [pathlib.Path.cwd(), *pathlib.Path.cwd().parents]
-        if config.in_dir(str(p)) or len(list(p.glob(".tmuxp.*")))
+        if in_dir(str(p)) or len(list(p.glob(".tmuxp.*")))
     )
     # CWD look one directory up
     choices += [
@@ -457,118 +484,177 @@ def config_file_completion(ctx, params, incomplete):
         for p in pathlib.Path.cwd().glob("*/.tmuxp.*")
     ]
 
-    # Project configs
-    choices += sorted((config_dir / c).stem for c in config.in_dir(str(config_dir)))
+    # Project workspace
+    choices += sorted((workspace_dir / c).stem for c in in_dir(str(workspace_dir)))
 
     return sorted(str(c) for c in choices if str(c).startswith(incomplete))
 
 
-@click.command(name="load", short_help="Load tmuxp workspaces.")
-@click.pass_context
-@click.argument(
-    "config",
-    type=ConfigPath(exists=True),
-    nargs=-1,
-    shell_complete=config_file_completion,
-)
-@click.option("-S", "socket_path", help="pass-through for tmux -S")
-@click.option("-L", "socket_name", help="pass-through for tmux -L")
-@click.option("-f", "tmux_config_file", help="pass-through for tmux -f")
-@click.option("-s", "new_session_name", help="start new session with new session name")
-@click.option("--yes", "-y", "answer_yes", help="yes", is_flag=True)
-@click.option(
-    "-d", "detached", help="Load the session without attaching it", is_flag=True
-)
-@click.option(
-    "-a",
-    "append",
-    help="Load configuration, appending windows to the current session",
-    is_flag=True,
-)
-@click.option(
-    "colors",
-    "-2",
-    flag_value=256,
-    default=True,
-    help="Force tmux to assume the terminal supports 256 colours.",
-)
-@click.option(
-    "colors",
-    "-8",
-    flag_value=88,
-    help="Like -2, but indicates that the terminal supports 88 colours.",
-)
-@click.option("--log-file", help="File to log errors/output to")
+def create_load_subparser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
+    workspace_files = parser.add_argument(
+        "workspace_files",
+        nargs="+",
+        metavar="workspace-file",
+        help="filepath to session or filename of session in tmuxp workspace directory",
+    )
+    parser.add_argument(
+        "-L",
+        dest="socket_name",
+        metavar="socket_name",
+        action="store",
+        help="passthru to tmux(1) -L",
+    )
+    parser.add_argument(
+        "-S",
+        dest="socket_path",
+        metavar="socket_path",
+        action="store",
+        help="passthru to tmux(1) -S",
+    )
+
+    tmux_config_file = parser.add_argument(
+        "-f",
+        dest="tmux_config_file",
+        metavar="tmux_config_file",
+        help="passthru to tmux(1) -f",
+    )
+
+    parser.add_argument(
+        "-s",
+        dest="new_session_name",
+        metavar="new_session_name",
+        help="start new session with new session name",
+    )
+    parser.add_argument(
+        "--yes",
+        "-y",
+        dest="answer_yes",
+        action="store_true",
+        help="always answer yes",
+    )
+    parser.add_argument(
+        "-d",
+        dest="detached",
+        action="store_true",
+        help="load the session without attaching it",
+    )
+    parser.add_argument(
+        "-a",
+        "--append",
+        dest="append",
+        action="store_true",
+        help="load workspace, appending windows to the current session",
+    )
+    colorsgroup = parser.add_mutually_exclusive_group()
+
+    colorsgroup.add_argument(
+        "-2",
+        dest="colors",
+        action="store_const",
+        const=256,
+        help="force tmux to assume the terminal supports 256 colours.",
+    )
+
+    colorsgroup.add_argument(
+        "-8",
+        dest="colors",
+        action="store_const",
+        const=88,
+        help="like -2, but indicates that the terminal supports 88 colours.",
+    )
+    parser.set_defaults(colors=None)
+
+    log_file = parser.add_argument(
+        "--log-file",
+        metavar="file_path",
+        action="store",
+        help="file to log errors/output to",
+    )
+
+    try:
+        import shtab
+
+        workspace_files.complete = shtab.FILE  # type: ignore
+        tmux_config_file.complete = shtab.FILE  # type: ignore
+        log_file.complete = shtab.FILE  # type: ignore
+    except ImportError:
+        pass
+
+    return parser
+
+
 def command_load(
-    ctx,
-    config,
-    socket_name,
-    socket_path,
-    tmux_config_file,
-    new_session_name,
-    answer_yes,
-    detached,
-    append,
-    colors,
-    log_file,
-):
-    """Load a tmux workspace from each CONFIG.
+    args: CLILoadNamespace,
+    parser: t.Optional[argparse.ArgumentParser] = None,
+) -> None:
+    """Load a tmux workspace from each WORKSPACE_FILE.
 
-    CONFIG is a specifier for a configuration file.
+    WORKSPACE_FILE is a specifier for a workspace file.
 
-    If CONFIG is a path to a directory, tmuxp will search it for
+    If WORKSPACE_FILE is a path to a directory, tmuxp will search it for
     ".tmuxp.{yaml,yml,json}".
 
-    If CONFIG is has no directory component and only a filename, e.g.
-    "myconfig.yaml", tmuxp will search the users's config directory for that
+    If WORKSPACE_FILE is has no directory component and only a filename, e.g.
+    "myworkspace.yaml", tmuxp will search the users's workspace directory for that
     file.
 
-    If CONFIG has no directory component, and only a name with no extension,
-    e.g. "myconfig", tmuxp will search the users's config directory for any
+    If WORKSPACE_FILE has no directory component, and only a name with no extension,
+    e.g. "myworkspace", tmuxp will search the users's workspace directory for any
     file with the extension ".yaml", ".yml", or ".json" that matches that name.
 
-    If multiple configuration files that match a given CONFIG are found, tmuxp
+    If multiple workspace files that match a given WORKSPACE_FILE are found, tmuxp
     will warn and pick the first one found.
 
-    If multiple CONFIGs are provided, workspaces will be created for all of
+    If multiple WORKSPACE_FILEs are provided, workspaces will be created for all of
     them. The last one provided will be attached. The others will be created in
     detached mode.
     """
     util.oh_my_zsh_auto_title()
 
-    if log_file:
-        logfile_handler = logging.FileHandler(log_file)
+    if args.log_file:
+        logfile_handler = logging.FileHandler(args.log_file)
         logfile_handler.setFormatter(log.LogFormatter())
         from . import logger
 
         logger.addHandler(logfile_handler)
 
     tmux_options = {
-        "socket_name": socket_name,
-        "socket_path": socket_path,
-        "tmux_config_file": tmux_config_file,
-        "new_session_name": new_session_name,
-        "answer_yes": answer_yes,
-        "colors": colors,
-        "detached": detached,
-        "append": append,
+        "socket_name": args.socket_name,
+        "socket_path": args.socket_path,
+        "tmux_config_file": args.tmux_config_file,
+        "new_session_name": args.new_session_name,
+        "answer_yes": args.answer_yes,
+        "colors": args.colors,
+        "detached": args.detached,
+        "append": args.append,
     }
 
-    if not config:
-        tmuxp_echo("Enter at least one CONFIG")
-        tmuxp_echo(ctx.get_help(), color=ctx.color)
-        ctx.exit()
+    if args.workspace_files is None or len(args.workspace_files) == 0:
+        tmuxp_echo("Enter at least one config")
+        if parser is not None:
+            parser.print_help()
+        sys.exit()
+        return
 
-    if isinstance(config, str):
-        load_workspace(config, **tmux_options)
+    last_idx = len(args.workspace_files) - 1
+    original_detached_option = tmux_options.pop("detached")
+    original_new_session_name = tmux_options.pop("new_session_name")
 
-    elif isinstance(config, tuple):
-        config = list(config)
-        # Load each configuration but the last to the background
-        for cfg in config[:-1]:
-            opt = tmux_options.copy()
-            opt.update({"detached": True, "new_session_name": None})
-            load_workspace(cfg, **opt)
+    for idx, workspace_file in enumerate(args.workspace_files):
+        workspace_file = find_workspace_file(
+            workspace_file, workspace_dir=get_workspace_dir()
+        )
 
-        # todo: obey the -d in the cli args only if user specifies
-        load_workspace(config[-1], **tmux_options)
+        detached = original_detached_option
+        new_session_name = original_new_session_name
+
+        if last_idx > 0 and idx < last_idx:
+            detached = True
+            new_session_name = None
+
+        load_workspace(
+            workspace_file,
+            detached=detached,
+            new_session_name=new_session_name,
+            **tmux_options,
+        )
